<?php if (!defined('PmWiki')) exit();
/**
 * qnotes.php
 * @author V.Krishn (vkrishn@insteps.net), Copyright 2010-2019
 * @note This file is part of Cookbook recipe "QNotes" for PmWiki;
 * You can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the  Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version. @see pmwiki.php for full details.
 *
 * @category pmwiki/cookbook
 * @package qnotes
 * @copyright Copyright (c) 2010-2019 V.Krishn (vkrishn@insteps.net)
 * @license GPL
 * @version 2.2.5-beta-2
 */

global $QNotes;
if ( ! IsEnabled($EnableQNotes, 0)) return;

## Current script variables
    if ( ! defined('DS') ) { define('DS', DIRECTORY_SEPARATOR); }
    $QNotes['Current']['dir'] = dirname(__FILE__);
    $EnableQNotesDebug = 0; # 0=no debug, 1=debug messages, 2=all
    $QNotes['debug']['msg'] = array();

## Get cookbook/modules meta and version variables
## @todo xml version of the metapage
    if (file_exists($QNotes['Current']['dir'] . DS . "version.php")) {
      include_once($QNotes['Current']['dir'] . DS . "version.php");
    }

## Get wiki group and pagename for use
    $_t = FmtPageName('$Group', $pagename);
    $QNotes['page']['CurrentGroup'] = $_t ? $_t : $DefaultGroup;
    $_t = FmtPageName('$Name', $pagename);
    $QNotes['page']['CurrentName'] = $_t ? $_t : $DefaultName;

## Set modules/cookbook install variables.
## Change these values accordingly.
    SDV($QNotes['Install']['location'], '0'); # 0=field, 1=farm, 2=others
    SDV($QNotes['Install']['path'], 'cookbook'); # relative to field or farm
    SDV($QNotes['Install']['dir'], 'qnotes'); # folder where installed
    SDV($QNotes['Install']['icon']['dir'], 'icons'); # icons to be used in qnotes

## Set default values of GROUP and PAGE for QNotes
    SDV($QNotes['default']['QNotesGroup'], 'QNotes');
    SDV($QNotes['default']['QNotesPageName'], 'QNotes1');

## Set page for qnotes application data
    SDV($QNotes['AppDataSetting'], $QNotes['default']['QNotesGroup'].'.AppDataSetting');

## Set default page to open for QNotes
    SDV($QNotes['DefaultPage'], $QNotes['default']['QNotesGroup'].'.'.$QNotes['default']['QNotesPageName']);

## Set HTMLPNewline for QNotes
    SDV($QNotes['HTMLPNewline'], '<br/>');

## To allow editing /w pass
    SDV($QNotes['qnotesIsAuthEdit'], true);
## To allow link resolving on page load
    SDV($QNotes['isResolveLinkOnPageLoad'], true);
## To post new node at end
    # defaults - only new root node is posted at end
    #          - new child nodes are posted at beginning (forum style)
    # allowed values - 1,3 (others @todo)
    #  1 - only new root node is posted at end
    #  3 - new root and child nodes are posted at end (section edit style)
    SDV($QNotes['isNewPostAtEnd'], 1);

## Allow directives in texts
    SDV($QNotes['isAllowDirectivesInText'], 0); # 0|1 - no|allow

## Error messages
    SDV($QNotes['qnotesPageNotFound'], '[--%red%link: page not found...%%--]');
    SDV($QNotes['qnotesNoteNotFound'], '[--%red%link: qnote not found...%%--]');
    SDV($QNotes['qnotesSelfLink'], '[--%red%link: selflink...%%--]');
    SDV($QNotes['qnotesRecursiveLink'], '%blue%link...%%');
    SDV($QNotes['qnotesLinkNoResolveOnPageLoad'], '%green%link...%%');
    SDV($QNotes['qnotesOPN_Msg'], 'Page Not Found...');

## Allow batch update, for importing into Qnotes
    SDV($QNotes['isBatchUpdate'], 0); # 0|1 - no|allow

## Use 'qnote' skin for async/ajax responses
    SDV($QNotes['isQnoteSkin'], 1); # 0|1 - no|allow


    // --------------------------------------
    // Would not require to edit below this
    // --------------------------------------


## Async/XmlHttpRequest
    include_once('scripts/functions.php');
    $is_xhr = $QNotes['isXhr'] = qnotesIsAjaxRequest();
    if($is_xhr && $QNotes['isQnoteSkin']) {
      global $Skin, $SkinLibDirs;

      $qnotesInstallDir = &$QNotes['Current']['dir'];
      $qnotesInstallUrlPath = &$QNotes['Install']['dirUrlPath'];
      $SkinLibDirs["\$qnotesInstallDir/skins/\$Skin"] = "\$qnotesInstallUrlPath/skins/\$Skin";
      $Skin = 'qnote';

      SDV($PageTextStartFmt, "\n");
      SDV($PageTextEndFmt, "\n");
    }


// //////////////////////////////////////////////////////////
//
// (:qnotes uniqueID qnoteDrawType qnotePage options :)
//    id - unique id
//    qnoteDrawType - list | editform | qnotes | link
//    qnotePage - qnotes page (default - <qnoteDefaultGroup>/<qnoteDefaultPage> )
//    options - options for list/editform/qnotes (see manual)
//
// //////////////////////////////////////////////////////////

# improves full page edit performance, but preview would not be available
if ($action == 'edit') { return; }

if(function_exists('Markup_e')) {
  # new format, no /e , IMPORTANT: pmwiki-2.2.58+ and php 5.5x
  Markup('qnotes','directives','/\\(:qnotes\\s(.*?):\\)/',"CallbackQnotes");
} else {
  /* old format, for pmwiki < 2.2.58
     if using php 5.5x see pmwiki documentation as how to suppress
       deprecated preg_replace errors. */
  Markup('qnotes','directives','/\\(:qnotes\\s(.*?):\\)/e',"qnotes('$1')");
}

function CallbackQnotes($m) {
  //extract($GLOBALS["MarkupToHTML"]); # to get $pagename
  return qnotes($m[1]);
}

$QNotes['qnotesDisplayMethod'] = PageTextVar($pagename,'qnotesDisplayMethod');
$QNotes['qnotesBehaviour'] = PageTextVar($pagename,'qnotesBehaviour');
$t = $QNotes['AppDataSetting'];
$FmtPV['$qnotesappsetting'] = "'$t'";
$HandleActions['qnotes'] = 'qnotesAsync';
$HandleAuth['qnotes'] = 'edit'; //@TODO
$FmtPV['$qnoteCategory'] = "'/default'";
$FmtPV['$qnoteTitle'] = "'#'";
$FmtPV['$qnoteText'] = "''";
SDVA($QNotes['formDataElement'], array(
     'qnotesCategory','qnotesTitle','qnotesText',
     'qnotesLinkToPgName','qnotesLinkTo','qnotesComment',
     'qnotesPage'));
$HandleActions['qnoteupgrade'] = 'qnotesUpgrade';
$QNotes['now'] = getdate(time());


if(strtolower($QNotes['qnotesBehaviour']) == 'sectionedit') {
 $QNotes['isNewPostAtEnd'] = isset($QNotes['isSectionEditNewPostAtEnd']) ?
       $QNotes['isSectionEditNewPostAtEnd'] : 3;
}

//Handle open qnotes page, set session variables
// for action, $_REQUEST['action'] = 'openQNotes'
if(strtolower(@$QNotes['qnotesDisplayMethod']) == 'single') {
  include_once('scripts/qnotes.pageopen.php');
}

//Handle upgrades
function qnotesUpgrade() {
  global $QNotes, $cookbook;
  if($_GET['qnupgrade'] == '1' && $_GET['qnupgradeto'] && $_POST['qnotesSubmit'] !== 'Save') {
    if (file_exists($QNotes['Current']['dir'] . DS . 'scripts/upgrade/qnotes.upgrade.php')) {
      include_once("scripts/upgrade/qnotes.upgrade.php");
    }
  }
  exit(0);
}

function qnotesAsyncRespose($t='') {
  global $FmtV, $HandleBrowseFmt, $PageStartFmt, $PageEndFmt, $PageRedirectFmt;
  global $Group, $Name, $pagename, $PageTextStartFmt, $PageTextEndFmt;
  $FmtV['$PageText'] = $t;
  SDV($HandleBrowseFmt, array(&$PageStartFmt, &$PageRedirectFmt, '$PageText',
    &$PageEndFmt));
  PrintFmt($pagename, $HandleBrowseFmt);
}

//Handle form async action/posting
function qnotesAsync() {
  global $QNotes, $Charset,
  $HTMLHeaderFmt, $HTMLFooterFmt,
  $EnableQNotesDebug;

  include_once('scripts/functions.php');
  include_once('scripts/get_set_qnotes.php');
  include_once('scripts/set_path_file.php');

  //Handle edit text request
  if($_GET['qn'] && $_GET['qnp'] && $_POST['qnotesSubmit'] !== 'Save' && $_GET['qfmt'] == 'wiki') { //@TODO
    include_once('scripts/qnotes.edit.php');
    return;
  }

  // updated node data (refresh request)
  if($_GET['async'] && $_GET['async'] == '1' && $_GET['qfmt'] == 'html') {
    include_once('scripts/qnotes.nodedata.php');
    return;
  }

  #reserved id for form posting //@TODO
  if(isset($_POST['qnotesFrmId']) && isset($_POST['qnotesSubmit'])) {
    include_once('scripts/qnotes.post.php');
    return;
  }

  return;
}

function qnotes($params) {
  global $QNotes, $ScriptUrl,
         $HTMLHeaderFmt, $HTMLFooterFmt,
         $EnableQNotesDebug;

  static $qnotesCount; $qnotesCount++;
  include_once('scripts/functions.php');
  include_once('scripts/get_set_qnotes.php');
  include_once('scripts/set_path_file.php');

  qnotes_init_lib();

  //shortname
  $qnotesInstallDir = &$QNotes['Current']['dir'];
  $qnotesInstallUrl = &$QNotes['Install']['dirUrl'];
  $qnotesInstallUrlPath = &$QNotes['Install']['dirUrlPath'];
  $qnotesIconDir = &$QNotes['Install']['icon']['dirFmt'];
  $qnotesIconUrl = &$QNotes['Install']['icon']['dirUrl'];
  $qnotesIconUrlPath = &$QNotes['Install']['icon']['dirUrlPath'];
  include_once('themes/themes.php');

  $args = ParseArgs(PSS($params));
  $count = count($args[""]);
  if($EnableQNotesDebug > 0) {
    $QNotes['args'][] = $args;
  }

  switch (strtolower(@$args[''][1])) {
    case 'qnotes':
      include_once('scripts/qnotes.notes.php');
      break;
    case 'list':
      include('scripts/qnotes.list.php');
      break;
    case 'editform':
      include_once('scripts/qnotes.editform.php');
      break;
    case 'link':
      include('scripts/qnotes.link.php');
      break;
    case 'import':
      include('scripts/qnotes.import.php');
      break;
    case 'createpage':
      include_once('scripts/qnotes.createpage.php');
      break;
  }

  switch (strtolower(@$args[''][0])) {
    case 'include':
      include('scripts/qnotes.include.php');
      break;
  }

  ## Debug
  $qnotesDebugMsg = QNotesDebug($EnableQNotesDebug);
  if($qnotesDebugMsg) {
    @$HTMLFooterFmt['qnotes_DEBUG'] =
        '<pre>' . $qnotesDebugMsg . '</pre>' . "\n";
  }

  return @$qnotesOutFmt;
  //return '<pre>' . print_r($a,true) . print_r($b,true) . '</pre>';

}




