<?php if (!defined('PmWiki')) exit();
/**
 * qnotes.import.php
 * @author V.Krishn (vkrishn@insteps.net), Copyright 2010-2019
 * @note This file is part of Cookbook recipe "QNotes" for PmWiki;
 * You can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the  Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version. @see pmwiki.php for full details.
 * @package qnotes
 */

// **********************************************************
// Import text/notes from other format
// **********************************************************
/*
  Directive
    (:qnotes id01 import <PageName> format='cherrytree'
        wrapfmt="0|1|2" template='Templates.QNotes'
        loglevel=0|1 prefix=<nodeprefix> :)
  
  Import file name format:
    name1--name2--name3--name<N>.txt
    ...
    name1--nameA--nameB--name<Z>.txt

      name1 - becomes Group.
      name2 - becomes Name.

  1. Trick to import all <name1> into single Group.
     Batch rename to prefix all files with 'name1--'
     eg.
     rename 's/(.*)?/name1--$1/' *.txt

      name1--name2--name3--name<N>.txt
      becomes...
      name1--name1--name2--name3--name<N>.txt

  2. Trick to import files to desired pagename.
     eg. pagename = Notes.Notes1

     (:qnotes id01 import Notes.Notes1 prefix='Notes--Notes1--Import' :)

*/

global $UploadDir, $UploadPrefixFmt, $UploadUrlFmt, $EnableUploadOverwrite,
       $EnableUpload, $EnableDirectDownload, $QNotes, $qnotesOutFmt;
if ( ! IsEnabled($EnableUpload, 0)) {
  $qnotesOutFmt = 'Uploads not enabled.'; return; }

  list($usec0, $sec0) = explode(' ', microtime());
  $qnPageName = &$args[''][2];
  $TimeFmt = '%b %d, %Y, %I:%M%P';
  $template = 'QNotes.Template';
  if ( isset($args['template'])) { $template = $args['template']; }

  $prefix = "";
  if ( isset($args['prefix'])) { $prefix = trim($args['prefix'],'-').'--'; }

  $txtprefix = ""; $txtsuffix = "";
  if ( isset($args['wrapfmt'])) {
    $wrapfmt = (int)$args['wrapfmt'];
    switch ($wrapfmt) {
      case 0:
        break;
      case 1:
        $txtprefix = "[@\n"; $txtsuffix = "\n@]";
        break;
      case 2:
        $txtprefix = "```\n[@\n"; $txtsuffix = "\n@]\n```";
        break;
    }

  }


  /* ----- Get uploaded files for $qnPageName ----- */
  $uploaddir = FmtPageName("$UploadDir$UploadPrefixFmt", $qnPageName);
  $dirp = @opendir($uploaddir);
  if (!$dirp) { $qnotesOutFmt = "%bold%Qnotes import:: ''$qnPageName'' not found."; return; }
  $filelist = array(); $size = 0;
  $matchext = '/\\.txt$/i';
  while (($file=readdir($dirp)) !== false) {
    if ($file{0} == '.') continue;
    if (@$matchext && !preg_match(@$matchext, $file)) continue;
    $filelist[$file] = $file;
    $size+=filesize("$uploaddir/$file");
  }
  closedir($dirp); natcasesort($filelist);
  if(count($filelist) < 1) { $qnotesOutFmt = "No files to import."; return; }
  if(250000 <= (int)$size) {
    $size = sprintf("%.2f", $size/1024);
    $qnotesOutFmt = "'''Import failed: '''Max 250kb allowed, Current total size is%red% $size%% kb. Try with less files.<br>"; return; }
  /* ---------------------------------------------- */
  //$u = '<pre>' . print_r($filelist, true) . '</pre>';
  //print_r($u); exit;


  /* ---------------------------------------------- */
  $out1 = array(); $out2 = array(); $out3 = 0; $pnOld = '';
  foreach($filelist as $file=>$x) {

    $updatedQNote = array();
    $name = PUE("$uploadurl$file");
    $stat = stat("$uploaddir/$file");
    $b = $stat['size'].$file.$stat['mtime'];
    $f = substr($file, 0, -4); 

    $f = $prefix.$f;
    $f = explode('--',$f);

    $name = isset($f[1]) ? ucfirst($f[1]) : ucfirst($f[0]);
    $qnpagename = $updatedQNote['qnotesPage'] = ucfirst($f[0]) . '/' . $name ;
    array_shift($f); array_shift($f);

    $qncategory = $updatedQNote['qnotesCategory'] = '/'.implode('/',$f);
    $qntitle = $updatedQNote['qnotesTitle'] = '#';

    $qnpagename = str_replace('.', '', $qnpagename);
    $pn = MakePageName($pagename, $qnpagename);
    $updatedQNote['qnotesPage'] = $pn = FmtPageName('$FullName', $pn);


    // Create/Update qnote data-page
    // ------------------------------
    if( ! PageExists($pn) ) {
      if($pnOld !== $pn) {
        $pnOld = $pn; $out1[] = "Creating Qnotes datapage: %maroon%".$pn.'%%<br>';
      }
      $QNotes['isDiff'] = 0;
      $QNotes['EditFunctions'] = array('PostPage');
      $msg = CreateQnotesPage($pn, $template, 2);
      if( $msg === 1 ) { $qnotesOutFmt = "'''Qnotes datepage exists.'''<br>"; return; }
      if( $msg === 2 ) { $qnotesOutFmt = "'''Qnotes datepage template does not exists.'''<br>"; return; }
    } else {
      $updatedQNote['qnotesText']  = $txtprefix.trim(file_get_contents($uploaddir.'/'.$x)).$txtsuffix;
      $QNotes['EditFunctions'] = array('PostPage');
      $QNotes['isDiff'] = 1;
      UpdateQnote($updatedQNote, 'update', 1);
      $out3++;
      $donetxt = "%green bold%Import done: Remove uploaded import text 
                   and comment or remove import directive.%%";
      $out2[] = number_format($stat['size']) . " bytes ... ''from:'' " . 
              $file . "\n<br>node:%maroon% $qncategory%% > %maroon%$pn%% " . '<br/>';

    }


  }

  list($usec1, $sec1) = explode(' ', microtime());
  $time = sprintf("%.2f", $sec1 + $usec1 - $sec0 - $usec0);
  $text = ''; if(count($out1) > 0) $text = '%green%Datapages created, refresh to import notes.%%';
  $qnotesOutFmt = implode("\n",$out1).implode("\n",$out2)
                  . "\n'''Total qnotes imported: '''"
                  . $out3." (in $time seconds).<br>$text $donetxt<br>";

  if ( isset($args['loglevel'])) {
    $loglevel = (int)$args['loglevel'];
    switch ($loglevel) {
      case 0: $qnotesOutFmt = ""; break;
      case 1:
        break;
      default:
        break;
    }
  }

  return;



